import 'dart:async';

import 'package:analyzer/dart/element/element.dart' as az;
import 'package:analyzer/dart/element/type.dart' as az;
import 'package:build/build.dart';
import 'package:demo/annotation/mirror.dart';
import 'package:matchable_builder/matchable_builder.dart';
import 'package:renderable/renderable.dart' as renderable;

class MirrorGenerator extends MatchableBuilder {
  MirrorGenerator(BuilderOptions options) : super(options);

  @override
  Map<String, List<String>> get buildExtensions => {
        ".dart": [".mirror.dart"],
      };

  @override
  Matcher get matcher => Matcher.and([
        ElementAnnotationMatcher<Mirror>(),
      ]);

  @override
  FutureOr<void> generate(List<az.Element> elements, BuildStep buildStep) {
    if (elements.isEmpty) {
      return null;
    }

    var library = elements.first.library;

    var libraryElement = renderable.LibraryElement(
      imports: library.imports.map(renderable.AnalyzerUtils.parseImportElement).toList(),
      exports: library.exports.map(renderable.AnalyzerUtils.parseExportElement).toList(),
      topLevelElements: elements.map(renderable.AnalyzerUtils.parseElement).toList(),
    );

    var outputId = buildStep.inputId.changeExtension(".mirror.dart");
    buildStep.writeAsString(outputId, libraryElement.render());
  }
}
